"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.Connection_suppressErrors = Connection_suppressErrors;
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/keys"));
var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/starts-with"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/map"));
var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/json/stringify"));
/**
 * will monkey patch the connection object
 * so that any errors thrown by subsequently installed handlers are caught and logged
 * (ex: connection.onHover(() => throw new Error('oops!')))
 * this prevents the LSP client, on the other end, to receive errors
 * which can sometimes cause error messages to pop-up uncontrollably
 *
 * @param conn
 */
function Connection_suppressErrors(conn) {
  for (const k of (0, _keys.default)(conn)) {
    if (!(0, _startsWith.default)(k).call(k, 'on')) {
      continue;
    } // only onHover, onCodeLens, etc?
    const v = conn[k];
    if (typeof v !== 'function') {
      continue;
    }
    conn[k] = (...args) => {
      const args2 = (0, _map.default)(args).call(args, arg => typeof arg === 'function' ? with_catch2(arg, (e, fargs) => {
        const data = {
          handler: k,
          handlerInstallParams: args,
          handlerExecParams: fargs,
          error: e + ''
        };
        const dd = (0, _stringify.default)(data, null, 2);
        conn.console.error(dd);
        return null;
      }) : arg);
      return v.apply(conn, args2);
    };
  }
}
function with_catch2(f, clause) {
  return (...args) => catch2(() => f(...args), e => clause(e, args));
  function catch2(f, clause2) {
    try {
      const res = f();
      if (typeof res?.then === 'function') {
        // promise
        return res.catch?.(clause2);
      }
      return res;
    } catch (e) {
      return clause2(e);
    }
  }
}